@@ -1,6 +1,8 @@
 import asyncio
+import itertools
+
 import time
-from typing import Any, Iterable, Optional
+from typing import Any, Iterable, Optional, Callable
 from aiohttp import ClientSession
 from contextlib import asynccontextmanager
 from ..decorators.core import async_retry
@@ -67,10 +69,10 @@ class ConcurrentRequester:
             max_fps: Optional[float] = None,
             timeout: float | None = None,
     ):
-        self.concurrency_limit = concurrency_limit
-        self.timeout = timeout
-        self.rate_limiter = RateLimiter(max_fps)
-        self.semaphore = asyncio.Semaphore(concurrency_limit)
+        self._concurrency_limit = concurrency_limit
+        self._timeout = timeout
+        self._rate_limiter = RateLimiter(max_fps)
+        self._semaphore = asyncio.Semaphore(concurrency_limit)
 
     @asynccontextmanager
     async def _get_session(self):
@@ -87,16 +89,16 @@ class ConcurrentRequester:
             **kwargs
     ) -> RequestResult:
         """发送单个请求"""
-        async with self.semaphore:
+        async with self._semaphore:
             try:
                 # todo: 速率限制也许需要优化
-                await self.rate_limiter.acquire()
+                await self._rate_limiter.acquire()
 
                 start_time = time.time()
 
                 async with session.request(
                         method, url,
-                        timeout=self.timeout,
+                        timeout=self._timeout,
                         **kwargs
                 ) as response:
                     data = await response.json()
@@ -142,6 +144,7 @@ class ConcurrentRequester:
             request_params: Iterable[dict[str, Any]],
             url: str,
             method: str = 'POST',
+            total_requests: Optional[int] = None,
             show_progress: bool = True
     ) -> tuple[list[RequestResult], Optional[ProgressTracker]]:
         """
@@ -151,51 +154,90 @@ class ConcurrentRequester:
             Tuple[list[RequestResult], Optional[ProgressTracker]]:
             请求结果列表和进度跟踪器（如果启用了进度显示）
         """
-        request_params = list(request_params)
-        requests_with_ids = list(enumerate(request_params))
+        # --------------------------------
+        # 如果没有提供total_requests，且需要显示进度，则计算总数
+        progress = None
+        if total_requests is None and show_progress:
+            # 创建迭代器的副本
+            request_params, params_for_counting = itertools.tee(request_params)
+            total_requests = sum(1 for _ in params_for_counting)
 
-        bar_config = ProgressBarConfig()
-        progress = ProgressTracker(
-            len(requests_with_ids),
-            concurrency=self.concurrency_limit,
-            config = bar_config) if show_progress else None
+        if show_progress and total_requests is not None:
+            progress = ProgressTracker(
+                total_requests,
+                concurrency=self._concurrency_limit,
+                config=ProgressBarConfig()
+            )
 
         async with self._get_session() as session:
-            tasks = {
-                asyncio.create_task(
-                    self._send_single_request(
-                        session=session,
-                        request_id=request_id,
-                        url=url,
-                        method=method,
-                        **params
-                    )
-                ): request_id
-                for request_id, params in requests_with_ids
-            }
-
-            results = []
-            for task in asyncio.as_completed(tasks):
-                try:
-                    result = await task
-                except Exception as e:
-                    request_id = tasks[task]
-                    result = RequestResult(
-                        request_id=request_id,
-                        data={
-                            'error': 'Task execution error',
-                            'error_type': e.__class__.__name__,
-                            'detail': str(e)
-                        },
-                        status='error',
-                        latency=0
-                    )
-
-                if progress:
-                    progress.update(result)
-                results.append(result)
+            results = await self.process_with_concurrency_window(
+                items=request_params,
+                process_func=lambda params, request_id: self._send_single_request(
+                    session=session,
+                    request_id=request_id,
+                    url=url,
+                    method=method,
+                    **params
+                ),
+                concurrency_limit=self._concurrency_limit,
+                progress=progress
+            )
 
             if progress:
                 progress.print_summary()
 
-            return sorted(results, key=lambda x: x.request_id), progress
\ No newline at end of file
+            return results, progress
+
+
+    async def process_with_concurrency_window(
+        self,
+        items: Iterable,
+        process_func: Callable,
+        concurrency_limit: int,
+        progress: Optional[ProgressTracker] = None,
+    ) -> list:
+        """
+        使用滑动窗口方式处理并发任务
+
+        Args:
+            items: 待处理的项目迭代器
+            process_func: 处理单个项目的异步函数，接收item和项目item_id作为参数
+            concurrency_limit: 并发限制数量,也是窗口大小
+            progress: 可选的进度跟踪器
+
+        Returns:
+            list[R]: 按照输入顺序返回的处理结果列表
+        """
+        results = []
+        item_id = 0
+        active_tasks = set()
+
+        for item in items:
+            # 如果活跃任务数达到并发限制，等待某个任务完成
+            if len(active_tasks) >= concurrency_limit:
+                done, active_tasks = await asyncio.wait(
+                    active_tasks,
+                    return_when=asyncio.FIRST_COMPLETED
+                )
+                # 处理完成的任务结果
+                for task in done:
+                    result = await task
+                    if progress:
+                        progress.update(result)
+                    results.append(result)
+
+            # 创建新任务
+            task = asyncio.create_task(process_func(item, item_id))
+            active_tasks.add(task)
+            item_id += 1
+
+        # 等待剩余任务完成
+        if active_tasks:
+            done, _ = await asyncio.wait(active_tasks)
+            for task in done:
+                result = await task
+                if progress:
+                    progress.update(result)
+                results.append(result)
+
+        return sorted(results, key=lambda x: getattr(x, 'request_id', 0))
